#!/bin/sh

_PGDATA=/var/services/pgsql
PGDATA="`/usr/bin/readlink ${_PGDATA}`"
PGDIRNAME="`/usr/bin/dirname ${PGDATA}`"
PGDATA83="${PGDIRNAME}/pgsql.83.`/bin/date +%s`"
PGMAJORVERSION=9.3
PREVMAJORVERSION=8.3
PGUPGRADE_FLAG="/tmp/.UpgradePGSQLDatabase"
ERR_MESSAGE="Failed to upgrade PostgreSQL, please contact Synology for support."

ChkPGSQLShare()
{
	/usr/syno/bin/servicetool --get-service-path pgsql >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "No volume to start PostgreSQL."
		return 1
	fi
	return 0
}

ChkExecEnv()
{
	if ! ChkPGSQLShare; then
		exit 1
	fi
	if [ ! -f /etc/postgresql/postgresql.conf ]; then
		/bin/cp -a /etc.defaults/postgresql /etc
	fi

	/bin/mkdir -p /run/postgresql
	/bin/chown -R postgres: /etc/postgresql /run/postgresql ${PGDATA}
	/bin/chmod 700 /etc/postgresql ${PGDATA}

	if [ ! -L ${PGDATA}/postgresql.conf ]; then
		/bin/ln -sf /etc/postgresql/postgresql.conf ${PGDATA}/postgresql.conf
	fi
}

ChkCompat()
{
	/usr/bin/pg_controldata --check-compatibility ${PGDATA} 2>/dev/null
	if [ $? -ne 0 ]; then
		echo "Compatibility check is failed. Please check /var/log/postgresql.log for further information."
		/bin/mv -f ${PGDATA} "${PGDIRNAME}/.pgsql.`/bin/date +%s`"
		InitDB
	fi
}

InitDB()
{
	local enable_checksum=""

	if [ "x`/bin/get_key_value /etc.defaults/synoinfo.conf support_postgresql_data_checksums`" == "xyes" ]; then
		enable_checksum="--data-checksums"
	fi

	if [ ! -d ${PGDATA}/base ]; then
		echo "Initialize PostgreSQL database"
		/bin/mkdir -p ${PGDATA}
		/bin/chown -R postgres: ${PGDATA}
		/bin/chmod 700 /etc/postgresql ${PGDATA}
		/bin/su postgres -c "/usr/bin/initdb --locale=POSIX --pgdata=${PGDATA} ${enable_checksum}" >/dev/null 2>&1
		/bin/ln -sf /etc/postgresql/postgresql.conf ${PGDATA}/postgresql.conf
		/bin/sync; /bin/sync; /bin/sync
	fi
}

UpgDB()
{
	/bin/mv ${PGDATA} ${PGDATA83}
	/bin/chown -R postgres: ${PGDATA83}

	InitDB

	if [ -f ${PGDATA83}/pgsql-8.3.sql -a -f ${PGDATA83}/pgsql-8.3.sql.md5 ]; then
		local md5sum="`/usr/syno/bin/openssl md5 ${PGDATA83}/pgsql-8.3.sql 2>/dev/null | /usr/bin/awk '{print $2}'`"
		if [ x`/bin/cat "${PGDATA83}/pgsql-8.3.sql.md5"` == x"$md5sum" ]; then
			ExecRestore
			return 0
		fi
	fi

	/usr/share/pgsql-8.3/bin/pg_controldata ${PGDATA83} > ${PGDATA83}/pg_controldata.log 2>&1
	if [ $? -ne 0 -o x`/bin/cat "${PGDATA83}/PG_VERSION"` != x"$PREVMAJORVERSION" ]; then
		/usr/bin/logger -p user.err -t `basename $0` "PostgreSQL database must be initialized with version ${PREVMAJORVERSION}."
		return 1
	fi

	/bin/cp -f /etc.defaults/postgresql/postgresql.conf ${PGDATA83}
	if [ -f /run/postgresql/postmaster.pid ]; then
		/usr/bin/logger -p user.err $ERR_MESSAGE
		return 1
	fi

	/bin/rm -f /run/postgresql/.s.* /run/postgresql/postmaster.pid
	/bin/su postgres -c "/usr/share/pgsql-8.3/bin/pg_ctl -s -D ${PGDATA83} -l ${PGDATA83}/pg_upgrade_server.log start -w -t 120"
	if /bin/grep "database files are incompatible with server" ${PGDATA83}/pg_upgrade_server.log; then
		/usr/bin/logger -p user.err $ERR_MESSAGE
		return 1
	fi

	/bin/su postgres -c "/usr/share/pgsql-8.3/bin/createuser -U admin -s postgres"
	/bin/su postgres -c "/usr/share/pgsql-8.3/bin/pg_dumpall --disable-triggers -f ${PGDATA83}/pgsql-8.3.sql > ${PGDATA83}/pg_upgrade_dump.log 2>&1"
	/bin/sed -i ${PGDATA83}/pgsql-8.3.sql \
		-e 's/^CREATE ROLE admin;$//g' \
		-e 's/^ALTER ROLE admin .*;$//g' \
		-e 's/OWNER = admin/OWNER = postgres/g' \
		-e 's/OWNER TO admin;$/OWNER TO postgres;/g' \
		-e 's/FROM admin;$/FROM postgres;/g' \
		-e 's/TO admin;$/TO postgres;/g'

	/bin/su postgres -c "/usr/share/pgsql-8.3/bin/pg_ctl stop"

	if [ -f ${PGDATA83}/pgsql-8.3.sql ]; then
		ExecRestore
	else
		/usr/bin/logger -p user.err $ERR_MESSAGE
		return 1
	fi
	return 0
}

ExecRestore()
{
	/bin/su postgres -c "/usr/bin/pg_ctl -s -D ${_PGDATA} start -w -t 120 -o '-F'"
	/bin/su postgres -c "/usr/bin/psql < ${PGDATA83}/pgsql-8.3.sql > ${PGDATA83}/pg_upgrade_restore.log 2>&1"
}


case $1 in
	start)
		if ! pidof postgres ; then #FIXME avoid start pgsql when pgsql is running
			ChkExecEnv
			if [ -f /var/.UpgradeBootup -a x`/bin/cat "${PGDATA}/PG_VERSION"` == x"$PREVMAJORVERSION" ]; then
				echo start > $PGUPGRADE_FLAG
				if ! UpgDB; then
					/usr/bin/logger -p user.err -t `basename $0` "Upgrade pgsql database failed."
					rm $PGUPGRADE_FLAG
					exit 1
				fi
				if [ "`cat $PGUPGRADE_FLAG`" = "timeout" ]; then
					/usr/bin/logger -p user.err -t `basename $0` "pgsql upgrade has been done but exceed 600 second."
					/usr/syno/bin/synodsmnotify @administrators dsmnotify:system_event widget:pgsql_upgrade_done
				fi
				rm $PGUPGRADE_FLAG
			elif [ -f /var/.PostgreSQLUpgrade ]; then
				ChkCompat
				/bin/su postgres -c "/usr/bin/pg_ctl -s -D ${_PGDATA} start -w -t 120 -o '-F'"

				if /usr/bin/psql -U postgres -l | grep -q '(3 rows)'; then
					echo start > $PGUPGRADE_FLAG

					_PGDATA83="`/bin/ls -1tl ${PGDIRNAME}/pgsql.83.*/pgsql-8.3.sql | /usr/bin/awk '{print $7}' | /usr/bin/tail -n 1`"
					PGDATA83="`/usr/bin/dirname ${_PGDATA83}`"
					/bin/su postgres -c "/usr/bin/psql < ${PGDATA83}/pgsql-8.3.sql > ${PGDATA83}/pg_upgrade_restore.log 2>&1"

					if [ "`cat $PGUPGRADE_FLAG`" = "timeout" ]; then
						/usr/bin/logger -p user.err -t `basename $0` "pgsql upgrade has been done but exceed 600 second."
						/usr/syno/bin/synodsmnotify @administrators dsmnotify:system_event widget:pgsql_upgrade_done
					fi

					rm $PGUPGRADE_FLAG
				fi

				rm /var/.PostgreSQLUpgrade
			else
				ChkCompat
				/bin/su postgres -c "/usr/bin/pg_ctl -s -D ${_PGDATA} start -w -t 120" > /run/postgresql/.PGSQL.log 2>&1
				if [ $? -ne 0 ]; then
					/bin/cat /run/postgresql/.PGSQL.log >> /var/log/postgresql.log
					/bin/echo "Failed to start PostgreSQL. Please check /var/log/postgresql.log"
					if /bin/grep "No space left on device" /run/postgresql/.PGSQL.log; then
						/usr/syno/bin/synodsmnotify @administrators dsmnotify:system_event widget:pgsql_startup_failed
						/usr/bin/logger -p user.err -t `basename $0` "no space left on device for starting PostgreSQL."
					fi
					/bin/rm -f /run/postgresql/.PGSQL.log
					exit 1
				fi
			fi

			#FIXME: workaround for database upgrade, we should remove this part after moveing `synologd' into service framework
			/usr/syno/etc/rc.d/S23synologd.sh restart
			/usr/syno/bin/mediaserver.sh start
		fi
		;;

	stop)
		if pidof postgres ; then #FIXME avoid stop pgsql when pgsql is not running
			ChkExecEnv
			/bin/su postgres -c "/usr/bin/pg_ctl -s -D ${_PGDATA} stop -m fast" >/dev/null 2>&1
		fi
		;;

	#restart) #FIXME
	reload|restart)
		ChkExecEnv
		ChkCompat
		/bin/su postgres -c "/usr/bin/pg_ctl -s -D ${_PGDATA} restart -m fast" >/dev/null 2>&1
		;;

#	reload)
#		ChkExecEnv
#		/bin/su postgres -c "/usr/bin/pg_ctl -s -D ${_PGDATA} reload" >/dev/null 2>&1
#		;;

	status)
		if [ ChkPGSQLShare -eq 1 ]; then 
			exit 1;
		fi
		/bin/su postgres -c "/usr/bin/pg_ctl -s -D ${_PGDATA} status" || exit 1
		;;

	*)
		echo "usage: `basename $0` {start|stop|restart|reload|status}" >&2
		exit 64
		;;
esac
